import { BlockPermutation, system, world } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe((data) => {
    data.blockTypeRegistry.registerCustomComponent('door:toggle', {
        onPlayerInteract: event => {
            const {block} = event;
            const isOpen = block.permutation.getState('door:is_open');
            const getHalfDoor = () => {
                switch(block.permutation.getState('door:part')) {
                    case 'lower': return block.above();
                    case 'upper': return block.below();
                }
            }
            const halfDoor = getHalfDoor();

            block.setPermutation(block.permutation.withState('door:is_open', !isOpen));
            block.dimension.playSound(isOpen ? 'random.door_close' : 'random.door_open', block.location, {pitch: 1.0, volume: 0.4});
            if(halfDoor?.permutation.matches(block.typeId)) {
                halfDoor.setPermutation(halfDoor.permutation.withState('door:is_open', !isOpen));
            }
        }
    });

    data.blockTypeRegistry.registerCustomComponent('door:on_place', {
        beforeOnPlayerPlace: event => {
            const {block, permutationToPlace} = event;
            const aboveBlock = block.above();
            const isCancelled = !aboveBlock.isAir && !aboveBlock.isLiquid;

            if(isCancelled) {
                event.cancel = true;
            }
            else {
                event.permutationToPlace = permutationToPlace.withState('door:part', 'lower');
                system.run(() => {
                    aboveBlock.setPermutation(BlockPermutation.resolve(block.typeId, {
                        ...block.permutation.getAllStates(),
                        'door:part': 'upper'
                    }));
                });
            }
        }
    });

    data.blockTypeRegistry.registerCustomComponent('door:changeable_texture', {
        beforeOnPlayerPlace: event => {
            const {player, permutationToPlace} = event;
            event.permutationToPlace = permutationToPlace.withState('door:top_texture', player.isSneaking);
        }
    });
});